************************************************************************
* This file is part of OpenMolcas.                                     *
*                                                                      *
* OpenMolcas is free software; you can redistribute it and/or modify   *
* it under the terms of the GNU Lesser General Public License, v. 2.1. *
* OpenMolcas is distributed in the hope that it will be useful, but it *
* is provided "as is" and without any express or implied warranties.   *
* For more details see the full text of the license in the file        *
* LICENSE or in <http://www.gnu.org/licenses/>.                        *
************************************************************************
      Subroutine SqM(a,b,n)
      integer n,i,j,ij
      Real*8 a(*),b(n,n)
      ij=0
      Do i=1,n
       Do j=i,n
        ij=ij+1
        B(i,j)=A(ij)
        B(j,i)=A(ij)
       End Do
      End Do
      Return
      End
