************************************************************************
* This file is part of OpenMolcas.                                     *
*                                                                      *
* OpenMolcas is free software; you can redistribute it and/or modify   *
* it under the terms of the GNU Lesser General Public License, v. 2.1. *
* OpenMolcas is distributed in the hope that it will be useful, but it *
* is provided "as is" and without any express or implied warranties.   *
* For more details see the full text of the license in the file        *
* LICENSE or in <http://www.gnu.org/licenses/>.                        *
************************************************************************
      Subroutine DecideOnLocalDF(LocalDF)
      Implicit None
      Logical LocalDF

      Logical DF
      Integer DFMode

      Call DecideOnDF(DF)
      If (DF) Then
         Call Get_iScalar('DF Mode',DFMode)
         LocalDF=DFMode.eq.1
      Else
         LocalDF=.False.
      End IF

      End
