************************************************************************
* This file is part of OpenMolcas.                                     *
*                                                                      *
* OpenMolcas is free software; you can redistribute it and/or modify   *
* it under the terms of the GNU Lesser General Public License, v. 2.1. *
* OpenMolcas is distributed in the hope that it will be useful, but it *
* is provided "as is" and without any express or implied warranties.   *
* For more details see the full text of the license in the file        *
* LICENSE or in <http://www.gnu.org/licenses/>.                        *
************************************************************************
      SUBROUTINE  SYMCM2(i1,i2,i3,i4,i5)
      Call SysAbendMsg('syscm2',
     & ' Entering dummy SYMCM2 is fatal for me',' ')
c Avoid unused argument warnings
      If (.False.) Then
         Call Unused_integer(i1)
         Call Unused_integer(i2)
         Call Unused_integer(i3)
         Call Unused_integer(i4)
         Call Unused_integer(i5)
      End If
      END
