************************************************************************
* This file is part of OpenMolcas.                                     *
*                                                                      *
* OpenMolcas is free software; you can redistribute it and/or modify   *
* it under the terms of the GNU Lesser General Public License, v. 2.1. *
* OpenMolcas is distributed in the hope that it will be useful, but it *
* is provided "as is" and without any express or implied warranties.   *
* For more details see the full text of the license in the file        *
* LICENSE or in <http://www.gnu.org/licenses/>.                        *
************************************************************************
      SUBROUTINE DecideOn1CCD(Do1CCD)

      Implicit Real*8 (a-h,o-z)
      Logical Do1CCD


      Call Get_iScalar('System BitSwitch',iOption)
*
*     This is a logical variable (logical function)
      Do1CCD=iAnd(iOption,2**12).Eq.2**12
*
      Return
      End
