************************************************************************
* This file is part of OpenMolcas.                                     *
*                                                                      *
* OpenMolcas is free software; you can redistribute it and/or modify   *
* it under the terms of the GNU Lesser General Public License, v. 2.1. *
* OpenMolcas is distributed in the hope that it will be useful, but it *
* is provided "as is" and without any express or implied warranties.   *
* For more details see the full text of the license in the file        *
* LICENSE or in <http://www.gnu.org/licenses/>.                        *
*                                                                      *
* Copyright (C) 2013, Thomas Bondo Pedersen                            *
************************************************************************
      Subroutine ChoRPA_Drv()
C
C     Thomas Bondo Pedersen (CTCC,UiO), July 2013.
C
C     Driver routine for CD/DF-based RPA.
C
      Implicit None

      Character*10 SecNam
      Parameter (SecNam='ChoRPA_Drv')

      Call RPA_Warn(3,SecNam//': not implemented yet')

      End
