************************************************************************
* This file is part of OpenMolcas.                                     *
*                                                                      *
* OpenMolcas is free software; you can redistribute it and/or modify   *
* it under the terms of the GNU Lesser General Public License, v. 2.1. *
* OpenMolcas is distributed in the hope that it will be useful, but it *
* is provided "as is" and without any express or implied warranties.   *
* For more details see the full text of the license in the file        *
* LICENSE or in <http://www.gnu.org/licenses/>.                        *
************************************************************************
      Logical Function LDF_X_IsSet()
      Implicit None
#include "localdf.fh"
      Integer LDF_Status
      Call Get_iScalar('LDF Status',LDF_Status)
      LDF_X_IsSet=LDF_Status.eq.LDF_Set
      End
