************************************************************************
* This file is part of OpenMolcas.                                     *
*                                                                      *
* OpenMolcas is free software; you can redistribute it and/or modify   *
* it under the terms of the GNU Lesser General Public License, v. 2.1. *
* OpenMolcas is distributed in the hope that it will be useful, but it *
* is provided "as is" and without any express or implied warranties.   *
* For more details see the full text of the license in the file        *
* LICENSE or in <http://www.gnu.org/licenses/>.                        *
*                                                                      *
* Copyright (C) 1996-2006, T. Thorsteinsson and D. L. Cooper           *
************************************************************************
      subroutine kotani_cvb(bikcof,ndet,ifns)
      implicit real*8 (a-h,o-z)
      dimension bikcof(ndet,ifns)
      dimension dum(1)

c
c Kotani spin functions (generated by Gram-Schmidt orthonormalization
c                        of the Rumer functions)
      call schmidtn_cvb(bikcof,ifns,dum,ndet,0)
      do 4000 k=1,ifns/2
      call dswap_(ndet,bikcof(1,k),1,bikcof(1,ifns+1-k),1)
4000  continue
      return
      end
